require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xscsrserversetup',
  dscmeta_resource_friendly_name: 'xSCSRServerSetup',
  dscmeta_resource_name: 'MSFT_xSCSRServerSetup',
  dscmeta_module_name: 'xSCSR',
  dscmeta_module_version: '1.3.0.0',
  docs: 'The DSC xSCSRServerSetup resource type.
         Automatically generated from version 1.3.0.0',
  features: ['simple_get_filter', 'canonicalize'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    dsc_analysisdatabaseserverinstance: {
      type: 'String',
      desc: 'Name of the SQL Server analysis database instance to install to.',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_installfolder: {
      type: 'Optional[String]',
      desc: 'Folder to install to.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_dwdatabasename: {
      type: 'Optional[String]',
      desc: 'Name of the usage data warehouse database that already exists, or that Setup will create.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_ensure: {
      type: "Enum['Present', 'Absent']",
      desc: 'An enumerated value that describes if the Service Reporting server is expected to be installed on the machine. Present {default}   Absent    ',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_databaseserverinstance: {
      type: 'String',
      desc: 'Name of the SQL Server database instance to install to.',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_repositorydatabasename: {
      type: 'Optional[String]',
      desc: 'Name of the usage repository database that already exists, or that Setup will create.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_usemicrosoftupdate: {
      type: "Optional[Enum['Yes', 'No']]",
      desc: 'Participation in Microsoft Update (yes or no).',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_stagingdatabasename: {
      type: 'Optional[String]',
      desc: 'Name of the usage staging database that already exists, or that Setup will create.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_sourcefolder: {
      type: 'Optional[String]',
      desc: 'Folder within the source path containing the source files for installation.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_databaseserver: {
      type: 'String',
      desc: 'Name of the server that is running SQL Server where the databases already exist, or where Setup will create them .',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_sendceipreports: {
      type: "Optional[Enum['Yes', 'No']]",
      desc: 'Participation in Customer Experience Improvement Program (yes or no).',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_sourcepath: {
      type: 'String',
      desc: 'UNC path to the root of the source files for installation.',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_analysisdatabasename: {
      type: 'Optional[String]',
      desc: 'Name of the analysis database that already exists, or that Setup will create.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_setupcredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Credential to be used to perform the installation.',
      behaviour: :parameter,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_analysisdatabaseserver: {
      type: 'String',
      desc: 'Name of the server that is running SQL Server where the analysis database already exists, or where Setup will create it.',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
