require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xscsrserverupdate',
  dscmeta_resource_friendly_name: 'xSCSRServerUpdate',
  dscmeta_resource_name: 'MSFT_xSCSRServerUpdate',
  dscmeta_module_name: 'xSCSR',
  dscmeta_module_version: '1.3.0.0',
  docs: 'The DSC xSCSRServerUpdate resource type.
         Automatically generated from version 1.3.0.0',
  features: ['simple_get_filter', 'canonicalize'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    dsc_sourcepath: {
      type: 'String',
      desc: 'UNC path to the root of the source files for installation.',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_ensure: {
      type: "Enum['Present', 'Absent']",
      desc: 'An enumerated value that describes if the update is expected to be installed on the machine. Present {default}   Absent    ',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_setupcredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Credential to be used to perform the installation.',
      behaviour: :parameter,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_sourcefolder: {
      type: 'Optional[String]',
      desc: 'Folder within the source path containing the source files for installation.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_update: {
      type: 'Optional[String]',
      desc: 'Display name of the update.',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
